( cd usr/lib ; rm -rf libcdio.so.6 )
( cd usr/lib ; ln -sf libcdio.so.6.0.0 libcdio.so.6 )
( cd usr/lib ; rm -rf libcdio.so )
( cd usr/lib ; ln -sf libcdio.so.6.0.0 libcdio.so )
( cd usr/lib ; rm -rf libiso9660.so.4 )
( cd usr/lib ; ln -sf libiso9660.so.4.0.0 libiso9660.so.4 )
( cd usr/lib ; rm -rf libiso9660.so )
( cd usr/lib ; ln -sf libiso9660.so.4.0.0 libiso9660.so )
( cd usr/lib ; rm -rf libcdio_cdda.so.0 )
( cd usr/lib ; ln -sf libcdio_cdda.so.0.0.0 libcdio_cdda.so.0 )
( cd usr/lib ; rm -rf libcdio_cdda.so )
( cd usr/lib ; ln -sf libcdio_cdda.so.0.0.0 libcdio_cdda.so )
( cd usr/lib ; rm -rf libcdio_paranoia.so.0 )
( cd usr/lib ; ln -sf libcdio_paranoia.so.0.0.0 libcdio_paranoia.so.0 )
( cd usr/lib ; rm -rf libcdio_paranoia.so )
( cd usr/lib ; ln -sf libcdio_paranoia.so.0.0.0 libcdio_paranoia.so )
